#!/usr/bin/python
#
# Displays tweets on LCD display
# HD44780 LCD Class for Raspberry Pi
#
# Based on code from: AndyPi (http://andypi.co.uk/)
# Based on work by : Matt Hawkins + Texy



import time
import pytz

import tweepy

CONSUMER_KEY = ''
CONSUMER_SECRET = ''
ACCESS_KEY = ''
ACCESS_SECRET = ''


auth = tweepy.OAuthHandler(CONSUMER_KEY, CONSUMER_SECRET)
auth.set_access_token(ACCESS_KEY, ACCESS_SECRET)
api = tweepy.API(auth)



tweets = api.user_timeline(count = 1)

tweet = tweets[0].text
while len(tweet) < 26:
	tweet = tweet + " "


#Adjust to local timezone
eastern = pytz.timezone('US/Eastern')
utc = pytz.timezone('UTC')
tweet_time = utc.normalize(utc.localize(tweets[0].created_at))
tweet_time_local = str(tweet_time.astimezone(eastern))


class RPi_LCD:

  # Define GPIO to LCD mapping
  LCD_RS = 7
  LCD_E  = 8
  LCD_D4 = 25
  LCD_D5 = 24
  LCD_D6 = 23
  LCD_D7 = 14

  # Define some device constants
  LCD_WIDTH = 16    # Maximum characters per line
  LCD_CHR = True
  LCD_CMD = False

  LCD_LINE_1 = 0x80 # LCD RAM address for the 1st line
  LCD_LINE_2 = 0xC0 # LCD RAM address for the 2nd line 

  # Timing constants
  E_PULSE = 0.00005
  E_DELAY = 0.00005
  
  import SafeGPIO as GPIO

  # Setup GPIO 
  GPIO.setup_pin(LCD_E, "out")  # E
  GPIO.setup_pin(LCD_RS, "out") # RS
  GPIO.setup_pin(LCD_D4, "out") # DB4
  GPIO.setup_pin(LCD_D5, "out") # DB5
  GPIO.setup_pin(LCD_D6, "out") # DB6
  GPIO.setup_pin(LCD_D7, "out") # DB7

  	
  def main(self):
    # Main function - Display latest tweet
    self.lcd_init()
    self.static_text(1,"l",tweet[0:16])
    self.static_text(2,"l",tweet[16:26] + " " + tweet_time_local[11:16])
    

  def lcd_init(self):
    # Initialise display
    self.lcd_byte(0x33,self.LCD_CMD)  # init
    self.lcd_byte(0x32,self.LCD_CMD)  # init
    self.lcd_byte(0x28,self.LCD_CMD)  # 2 line 5x7 matrix
    self.lcd_byte(0x0C,self.LCD_CMD)  # turn cursor off (0x0E to enable)
    self.lcd_byte(0x06,self.LCD_CMD)  # shift cursor right
    self.lcd_byte(0x01,self.LCD_CMD)  # clear display

  def cls(self):
    self.lcd_byte(0x01,self.LCD_CMD)  # clear display

  def static_text(self,line,just,message):
    # set justifitication
    if just=="l":
      message = message.ljust(self.LCD_WIDTH," ")

    elif just=="r":
      message = message.rjust(self.LCD_WIDTH," ")

    elif just=="c":
      message = message.center(self.LCD_WIDTH," ")
    
    time.sleep(0.01)
    # set which line to print to
    if line==1:line=self.LCD_LINE_1
    elif line==2:line=self.LCD_LINE_2
    self.lcd_byte(line,self.LCD_CMD)
    # send string to display
    for i in range(self.LCD_WIDTH):
      self.lcd_byte(ord(message[i]),self.LCD_CHR)
    


  def lcd_byte(self,bits, mode):
    # Send byte to data pins
    # bits = data
    # mode = True  for character
    #        False for command

    self.GPIO.gpio_output(self.LCD_RS, mode) # RS

    # High bits
    self.GPIO.gpio_output(self.LCD_D4, False)
    self.GPIO.gpio_output(self.LCD_D5, False)
    self.GPIO.gpio_output(self.LCD_D6, False)
    self.GPIO.gpio_output(self.LCD_D7, False)
    if bits&0x10==0x10:
      self.GPIO.gpio_output(self.LCD_D4, True)
    if bits&0x20==0x20:
      self.GPIO.gpio_output(self.LCD_D5, True)
    if bits&0x40==0x40:
      self.GPIO.gpio_output(self.LCD_D6, True)
    if bits&0x80==0x80:
      self.GPIO.gpio_output(self.LCD_D7, True)

    # Toggle 'Enable' pin
    time.sleep(self.E_DELAY)    
    self.GPIO.gpio_output(self.LCD_E, True)  
    time.sleep(self.E_PULSE)
    self.GPIO.gpio_output(self.LCD_E, False)  
    time.sleep(self.E_DELAY)      

    # Low bits
    self.GPIO.gpio_output(self.LCD_D4, False)
    self.GPIO.gpio_output(self.LCD_D5, False)
    self.GPIO.gpio_output(self.LCD_D6, False)
    self.GPIO.gpio_output(self.LCD_D7, False)
    if bits&0x01==0x01:
      self.GPIO.gpio_output(self.LCD_D4, True)
    if bits&0x02==0x02:
      self.GPIO.gpio_output(self.LCD_D5, True)
    if bits&0x04==0x04:
      self.GPIO.gpio_output(self.LCD_D6, True)
    if bits&0x08==0x08:
      self.GPIO.gpio_output(self.LCD_D7, True)

    # Toggle 'Enable' pin
    time.sleep(self.E_DELAY)    
    self.GPIO.gpio_output(self.LCD_E, True)  
    time.sleep(self.E_PULSE)
    self.GPIO.gpio_output(self.LCD_E, False)  
    time.sleep(self.E_DELAY)   

if __name__ == '__main__':
  lcd=RPi_LCD()
  lcd.main()
